@@ -1,30 +1,30 @@
-#define __GNOME_PRINT_CONFIG_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * And frontend abstraction to whatever config system we eventually have
+ *  gnome-print-config.c: And frontend abstraction to whatever config system we eventually have
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Authors:
- *   Lauris Kaplinski <lauris@helixcode.com>
+ *  Authors:
+ *    Lauris Kaplinski <lauris@helixcode.com>
  *
- * Copyright 2001 Ximian, Inc.
+ *  Copyright 2001 Ximian, Inc.
  *
  */
 
+#define __GNOME_PRINT_CONFIG_C__
+
 #include <ctype.h>
 #include <stdlib.h>
 #include <locale.h>
@@ -82,6 +82,21 @@ gnome_print_config_unref (GnomePrintConfig *config)
 	return NULL;
 }
 
+GnomePrintConfig *
+gnome_print_config_dup (GnomePrintConfig *old_config)
+{
+	GnomePrintConfig *config = NULL;
+
+	g_return_val_if_fail (old_config != NULL, NULL);
+
+	config = g_new (GnomePrintConfig, 1);
+
+	config->refcount = 1;
+	config->node = gpa_node_duplicate (old_config->node);
+
+	return config;
+}
+
 guchar *
 gnome_print_config_get (GnomePrintConfig *config, const guchar *key)
 {
@@ -93,8 +108,6 @@ gnome_print_config_get (GnomePrintConfig *config, const guchar *key)
 
 	val = gpa_node_get_path_value (config->node, key);
 
-	g_log ("GPC", G_LOG_LEVEL_DEBUG, "GET %s: %s : %s", (val) ? "OK" : "__", key, val);
-
 	return val;
 }
 
@@ -110,8 +123,6 @@ gnome_print_config_set (GnomePrintConfig *config, const guchar *key, const gucha
 
 	result = gpa_node_set_path_value (config->node, key, value);
 
-	g_log ("GPC", G_LOG_LEVEL_DEBUG, "PUT %s: %s : %s", (result) ? "OK" : "__", key, value);
-
 	return result;
 }
 
@@ -198,12 +209,13 @@ gboolean
 gnome_print_config_get_length (GnomePrintConfig *config, const guchar *key, gdouble *val, const GnomePrintUnit **unit)
 {
 	guchar *v;
+	const GnomePrintUnit *c_unit;
 
 	g_return_val_if_fail (config != NULL, FALSE);
 	g_return_val_if_fail (key != NULL, FALSE);
 	g_return_val_if_fail (*key != '\0', FALSE);
 	g_return_val_if_fail (val != NULL, FALSE);
-	g_return_val_if_fail (unit != NULL, FALSE);
+
 
 	v = gnome_print_config_get (config, key);
 
@@ -219,13 +231,20 @@ gnome_print_config_get_length (GnomePrintConfig *config, const guchar *key, gdou
 		}
 		while (*e && !isalnum (*e)) e++;
 		if (!*e) {
-			*unit = GNOME_PRINT_PS_UNIT;
+			c_unit = GNOME_PRINT_PS_UNIT;
 		} else {
-			*unit = gnome_print_unit_get_by_abbreviation (e);
-			if (!*unit) *unit = gnome_print_unit_get_by_name (e);
+			c_unit = gnome_print_unit_get_by_abbreviation (e);
+			if (!c_unit) c_unit = gnome_print_unit_get_by_name (e);
 		}
 		g_free (v);
-		return (*unit != NULL);
+		if (c_unit == NULL)
+			return FALSE;
+		if (unit != NULL) {
+			*unit = c_unit;
+		} else {
+			gnome_print_convert_distance (val, c_unit, GNOME_PRINT_PS_UNIT);
+		}
+		return TRUE;
 	}
 	
 	return FALSE;
@@ -297,3 +316,10 @@ gnome_print_config_set_length (GnomePrintConfig *config, const guchar *key, gdou
 }
 
 
+void
+gnome_print_config_dump (GnomePrintConfig *config)
+{
+	g_return_if_fail (config != NULL);
+
+	gpa_utils_dump_tree (config->node);
+}
