@@ -1,7 +1,6 @@
-#define __GNOME_PRINT_MASTER_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  A system print interface.
+ *  gnome-print-master.c: A system print interface.
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -24,18 +23,23 @@
  *  Copyright (C) 2000-2001 Ximian Inc.
  */
 
+#define __GNOME_PRINT_MASTER_C__
+
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <locale.h>
-#include "gpa-private.h"
+
+#include <libart_lgpl/art_affine.h>
+
+#include <gpa/gpa-private.h>
 #include "gnome-print-private.h"
 #include "gnome-print-meta.h"
 #include "gnome-print-multipage.h"
 #include "gnome-print-master-private.h"
 
-#define GPM_VERBOSE
+#define noGPM_VERBOSE
 
 typedef struct _GPMPrivate GPMPrivate;
 
@@ -191,7 +195,8 @@ gnome_print_master_get_config (GnomePrintMaster *gpm)
 	g_return_val_if_fail (gpm != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), NULL);
 
-	if (gpm->config) gnome_print_config_ref (gpm->config);
+	if (gpm->config)
+		gnome_print_config_ref (gpm->config);
 
 	return gpm->config;
 }
@@ -212,7 +217,8 @@ gnome_print_master_get_context (GnomePrintMaster *gpm)
 	g_return_val_if_fail (gpm != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), NULL);
 
-	if (gpm->meta) g_object_ref (G_OBJECT (gpm->meta));
+	if (gpm->meta)
+		g_object_ref (G_OBJECT (gpm->meta));
 
 	return gpm->meta;
 }
@@ -237,7 +243,8 @@ gnome_print_master_get_pages (GnomePrintMaster *gpm)
 	g_return_val_if_fail (gpm != NULL, 0);
 	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), 0);
 
-	if (!GNOME_PRINT_MASTER_CLOSED (gpm)) return 0;
+	if (!GNOME_PRINT_MASTER_CLOSED (gpm))
+		return 0;
 
 	gpm_update_layout_data (gpm);
 
@@ -245,7 +252,8 @@ gnome_print_master_get_pages (GnomePrintMaster *gpm)
 
 	mp = gnome_print_meta_get_pages (GNOME_PRINT_META (gpm->meta));
 
-	if (pp->num_affines <= 1) return mp;
+	if (pp->num_affines <= 1)
+		return mp;
 
 	return (mp + pp->num_affines - 1) / pp->num_affines;
 }
@@ -271,112 +279,28 @@ gnome_print_master_get_page_size_from_config (GnomePrintConfig *config, gdouble
 
 	if (pp->LY_LIST) {
 		/* Success */
-		if (width) *width = pp->LW;
-		if (height) *height = pp->LH;
+		if (width)
+			*width = pp->LW;
+		if (height)
+			*height = pp->LH;
 	} else {
-		if (width) *width = pp->pw;
-		if (height) *height = pp->ph;
+		if (width)
+			*width = pp->pw;
+		if (height)
+			*height = pp->ph;
 	}
 
 	return TRUE;
 }
 
-/**
- * gnome_print_master_set_paper:
- * @gpm: An initialised GnomePrintMaster.
- * @paper: Paper size required.
- * 
- * Set the paper size for the print.
- **/
-
-gint
-gnome_print_master_set_paper (GnomePrintMaster *gpm, const guchar *paper)
-{
-	gboolean ret;
-
-	g_return_val_if_fail (gpm != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (paper != NULL, GNOME_PRINT_ERROR_BADVALUE);
-
-	ret = gnome_print_config_set (gpm->config, GNOME_PRINT_KEY_PAPER_SIZE, paper);
-
-	return ret ? GNOME_PRINT_OK : GNOME_PRINT_ERROR_BADVALUE;
-}
-
-const guchar *
-gnome_print_master_get_paper (const GnomePrintMaster *gpm)
-{
-	g_return_val_if_fail (gpm != NULL, NULL);
-	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), NULL);
-
-	/* fixme: memleak */
-	return gnome_print_config_get (gpm->config, GNOME_PRINT_KEY_PAPER_SIZE);
-}
-
-/**
- * gnome_print_master_set_printer:
- * @gpm: An initialised GnomePrintMaster.
- * @printer: Printer device to use.
- * 
- * Sets physical printer used to print to.  If this is not set,
- * then a dialogue will be presented on each print.
- *
- * This functions takes ownership of the printer reference.
- **/
-
-gint
-gnome_print_master_set_printer (GnomePrintMaster *gpm, const guchar *printer)
-{
-	gboolean ret;
-
-	g_return_val_if_fail (gpm != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (printer != NULL, GNOME_PRINT_ERROR_BADVALUE);
-
-	ret = gnome_print_config_set (gpm->config, "Printer", printer);
-
-	return ret ? GNOME_PRINT_OK : GNOME_PRINT_ERROR_BADVALUE;
-}
-
-/**
- * gnome_print_master_set_copies:
- * @gpm: A GnomePrintContext.
- * @copies: Number of copies of each page to print.
- * @iscollate: Whether page copies are collated.
- * 
- * Set the number of copies to print, and whether pages are collated.
- * If @iscollate is set, then a multiple copy print to a physical
- * printer will order pages as "1,2,3,...1,2,3,..." otherwise they
- * will be ordered "1,1,...,2,2...".
- **/
-
-gint
-gnome_print_master_set_copies (GnomePrintMaster *gpm, gint copies, gboolean collate)
-{
-	gboolean ret;
-
-	g_return_val_if_fail (gpm != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), GNOME_PRINT_ERROR_UNKNOWN);
-
-	/* fixme: key */
-	ret = gnome_print_config_set_int (gpm->config, GNOME_PRINT_KEY_NUM_COPIES, copies);
-	g_return_val_if_fail (ret, GNOME_PRINT_ERROR_UNKNOWN);
-	/* fixme: key */
-	ret = gnome_print_config_set_boolean (gpm->config, GNOME_PRINT_KEY_COLLATE, collate);
-	g_return_val_if_fail (ret, GNOME_PRINT_ERROR_UNKNOWN);
-
-	return GNOME_PRINT_OK;
-}
-
 /**
  * gnome_print_master_close:
  * @gpm: A GnomePrintMaster which has had printing performed
- * on it.
  * 
- * Closes the GnomePrintMaster @gpm, ready for printing
- * or previewing.
+ *  * Closes the GnomePrintMaster @gpm, ready for printing or previewing.
+ * 
+ * Return Value: 
  **/
-
 gint
 gnome_print_master_close (GnomePrintMaster *gpm)
 {
@@ -431,7 +355,8 @@ gnome_print_master_print (GnomePrintMaster *gpm)
 
 	/* Get number of pages in metafile */
 	lpages = gnome_print_meta_get_pages (GNOME_PRINT_META (gpm->meta));
-	if (lpages < 1) return GNOME_PRINT_OK;
+	if (lpages < 1)
+		return GNOME_PRINT_OK;
 	npages = lpages;
 
 	/* Update layout data */
@@ -616,9 +541,9 @@ static void
 gpm_update_layout_data (GnomePrintMaster *gpm)
 {
 	GPMPrivate *pp;
-	ArtDRect area, r, PPDC;
-	gdouble xscale, yscale, scale, t;
-	gdouble PPDP2C[6], a[6], b[6];
+	ArtDRect area, r;
+	gdouble t;
+	gdouble a[6];
 	gint i;
 
 	g_return_if_fail (gpm->priv);
@@ -629,8 +554,10 @@ gpm_update_layout_data (GnomePrintMaster *gpm)
 
 	/* Now comes the fun part */
 
-	if (pp->num_affines < 1) return;
-	if ((fabs (pp->pw) < EPSILON) || (fabs (pp->ph) < EPSILON)) return;
+	if (pp->num_affines < 1)
+		return;
+	if ((fabs (pp->pw) < EPSILON) || (fabs (pp->ph) < EPSILON))
+		return;
 
 	/* Initial setup */
 	/* Calculate PP2PA */
@@ -654,7 +581,8 @@ gpm_update_layout_data (GnomePrintMaster *gpm)
 	art_drect_affine_transform (&r, &area, pp->PP2PA);
 	pp->PAW = r.x1 - r.x0;
 	pp->PAH = r.y1 - r.y0;
-	if ((pp->PAW < EPSILON) || (pp->PAH < EPSILON)) return;
+	if ((pp->PAW < EPSILON) || (pp->PAH < EPSILON))
+		return;
 
 	/* Now we have to find the size of layout page */
 	/* Again, knowing that layouts are rectilinear helps us */
@@ -682,7 +610,8 @@ gpm_update_layout_data (GnomePrintMaster *gpm)
 	art_drect_affine_transform (&r, &area, a);
 	pp->LW = r.x1 - r.x0;
 	pp->LH = r.y1 - r.y0;
-	if ((pp->LW < EPSILON) || (pp->LH < EPSILON)) return;
+	if ((pp->LW < EPSILON) || (pp->LH < EPSILON))
+		return;
 	PRINT_2 ("L Dimensions", pp->LW, pp->LH);
 	/* Now complete matrix calculation */
 	t = pp->LW * pp->LP2LY[0] + pp->LH * pp->LP2LY[2];
@@ -755,29 +684,26 @@ gpm_parse_config_data (GnomePrintMaster *gpm)
 	if (layout) {
 		gint numlp;
 		numlp = 0;
-		g_print ("Got layout\n");
 		if (gpa_node_get_int_path_value (layout, "LogicalPages", &numlp) && (numlp > 0)) {
 			GPANode *pages;
-			g_print ("Got numlp %d\n", numlp);
 			pages = gpa_node_get_path_node (layout, "Pages");
 			if (pages) {
 				GPANode *page;
 				gdouble *affines;
 				gint pagenum;
-				g_print ("Got pages\n");
 				affines = g_new (gdouble, 6 * numlp);
 				pagenum = 0;
 				for (page = gpa_node_get_child (pages, NULL); page != NULL; page = gpa_node_get_child (pages, page)) {
 					guchar *transform;
-					gboolean ret;
 					transform = gpa_node_get_value (page);
-					g_print ("Got transform value %s\n", transform);
 					gpa_node_unref (page);
-					if (!transform) break;
+					if (!transform)
+						break;
 					gpm_parse_transform (transform, affines + 6 * pagenum);
 					g_free (transform);
 					pagenum += 1;
-					if (pagenum >= numlp) break;
+					if (pagenum >= numlp)
+						break;
 				}
 				gpa_node_unref (pages);
 				if (pagenum == numlp) {
@@ -827,12 +753,17 @@ gpm_clear_config_data (GnomePrintMaster *gpm)
 	}
 }
 
+
+#ifdef __GNUC__
+#warning This function should not be here, we should not be including gpa-private.h too
+#endif
 /* All keys can be NULL */
-GnomePrintLayoutData *gnome_print_config_get_layout_data (GnomePrintConfig *config,
-							  const guchar *pagekey,
-							  const guchar *porientkey,
-							  const guchar *lorientkey,
-							  const guchar *layoutkey)
+GnomePrintLayoutData *
+gnome_print_config_get_layout_data (GnomePrintConfig *config,
+				    const guchar *pagekey,
+				    const guchar *porientkey,
+				    const guchar *lorientkey,
+				    const guchar *layoutkey)
 {
 	GnomePrintLayoutData *lyd;
 	guchar key[1024];
@@ -847,10 +778,14 @@ GnomePrintLayoutData *gnome_print_config_get_layout_data (GnomePrintConfig *conf
 
 	g_return_val_if_fail (config != NULL, NULL);
 
-	if (!pagekey) pagekey = GNOME_PRINT_KEY_PAPER_SIZE;
-	if (!porientkey) porientkey = GNOME_PRINT_KEY_PAPER_ORIENTATION;
-	if (!lorientkey) lorientkey = GNOME_PRINT_KEY_PAGE_ORIENTATION;
-	if (!layoutkey) layoutkey = GNOME_PRINT_KEY_LAYOUT;
+	if (!pagekey)
+		pagekey = GNOME_PRINT_KEY_PAPER_SIZE;
+	if (!porientkey)
+		porientkey = GNOME_PRINT_KEY_PAPER_ORIENTATION;
+	if (!lorientkey)
+		lorientkey = GNOME_PRINT_KEY_PAGE_ORIENTATION;
+	if (!layoutkey)
+		layoutkey = GNOME_PRINT_KEY_LAYOUT;
 
 
 	/* Initialize */
@@ -894,28 +829,25 @@ GnomePrintLayoutData *gnome_print_config_get_layout_data (GnomePrintConfig *conf
 	if (layout) {
 		gint numlp;
 		numlp = 0;
-		g_print ("Got layout\n");
 		if (gpa_node_get_int_path_value (layout, "LogicalPages", &numlp) && (numlp > 0)) {
 			GPANode *pnodes;
-			g_print ("Got numlp %d\n", numlp);
 			pnodes = gpa_node_get_path_node (layout, "Pages");
 			if (pnodes) {
 				GPANode *page;
 				gint pagenum;
-				g_print ("Got pages\n");
 				pages = g_new (GnomePrintLayoutPageData, numlp);
 				pagenum = 0;
 				for (page = gpa_node_get_child (pnodes, NULL); page != NULL; page = gpa_node_get_child (pnodes, page)) {
 					guchar *transform;
-					gboolean ret;
 					transform = gpa_node_get_value (page);
-					g_print ("Got transform value %s\n", transform);
 					gpa_node_unref (page);
-					if (!transform) break;
+					if (!transform)
+						break;
 					gpm_parse_transform (transform, pages[pagenum].matrix);
 					g_free (transform);
 					pagenum += 1;
-					if (pagenum >= numlp) break;
+					if (pagenum >= numlp)
+						break;
 				}
 				gpa_node_unref (pnodes);
 				if (pagenum == numlp) {
@@ -951,7 +883,8 @@ gnome_print_layout_data_free (GnomePrintLayoutData *lyd)
 {
 	g_return_if_fail (lyd != NULL);
 
-	if (lyd->pages) g_free (lyd->pages);
+	if (lyd->pages)
+		g_free (lyd->pages);
 	g_free (lyd);
 }
 
@@ -959,10 +892,9 @@ GnomePrintLayout *
 gnome_print_layout_new_from_data (const GnomePrintLayoutData *lyd)
 {
 	GnomePrintLayout *ly;
-	GnomePrintLayout ll;
-	ArtDRect area, r, PPDC;
-	gdouble xscale, yscale, scale, t;
-	gdouble PPDP2C[6], a[6], b[6];
+	ArtDRect area, r;
+	gdouble t;
+	gdouble a[6];
 	gint i;
 	/* Layout data */
 	gdouble PP2PA[6], LP2LY[6];
@@ -1071,7 +1003,8 @@ gnome_print_layout_free (GnomePrintLayout *layout)
 {
 	g_return_if_fail (layout != NULL);
 
-	if (layout->LYP) g_free (layout->LYP);
+	if (layout->LYP)
+		g_free (layout->LYP);
 	g_free (layout);
 }
 
@@ -1079,16 +1012,19 @@ static gboolean
 gpm_parse_transform (guchar *str, gdouble *transform)
 {
 	gdouble t[6];
-	gchar *p, *e;
+	guchar *p;
+	gchar *e;
 	gint i;
 
 	art_affine_identity (transform);
 
 	p = str;
 	p = strchr (str, '(');
-	if (!p) return FALSE;
+	if (!p)
+		return FALSE;
 	p += 1;
-	if (!*p) return;
+	if (!*p)
+		return FALSE;
 	for (i = 0; i < 6; i++) {
 		while (*p && isspace (*p)) p += 1;
 		if (!strncmp (p, "SQRT2", 5)) {
@@ -1106,16 +1042,43 @@ gpm_parse_transform (guchar *str, gdouble *transform)
 		} else {
 			t[i] = strtod (p, &e);
 		}
-		if (e == p) return FALSE;
+		if (e == (gchar *) p)
+			return FALSE;
 		p = e;
 	}
 
 	memcpy (transform, t, 6 * sizeof (gdouble));
-	PRINT_AFFINE ("Got affine:", transform);
 
 	return TRUE;
 }
 
+
+/**
+ * gnome_print_master_set_print_to_file:
+ * @gmp: job
+ * @output: output file, if NULL sets print to file to FALSE
+ * 
+ * Sets/unsets the print to file option of the job 
+ * 
+ * Return Value: 
+ **/
+gint
+gnome_print_master_print_to_file (GnomePrintMaster *gpm, gchar *output)
+{
+	if (output) {
+		gnome_print_config_set (gpm->config, "Settings.Transport.Backend",    "file");
+		gnome_print_config_set (gpm->config, GNOME_PRINT_KEY_OUTPUT_FILENAME, output);
+	} else {
+		g_print ("Setting print to file to off, using LPR\n");
+		/* In the future we might want to use the default or even better,
+		 * go back to the prev. selected printer
+		 */
+		gnome_print_config_set (gpm->config, "Settings.Transport.Backend", "lpr");
+	}
+	
+	return GNOME_PRINT_OK;
+}
+
 /* fixme: Move this (and previous) to gnome-print-config.c (Lauris) */
 
 gboolean
@@ -1132,7 +1095,8 @@ gnome_print_config_get_transform (GnomePrintConfig *config, const guchar *key, g
 	g_return_val_if_fail (config != NULL, FALSE);
 
 	v = gnome_print_config_get (config, key);
-	if (!v) return FALSE;
+	if (!v)
+		return FALSE;
 
 	loc = setlocale (LC_NUMERIC, NULL);
 	setlocale (LC_NUMERIC, "C");
