@@ -1,13 +1,30 @@
-#define GP_PATH_C
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * GPPath
+ *  gp-path.c:
  *
- * (C) 1999-2000 Lauris Kaplinski <lauris@ariman.ee>
- * Released under LGPL
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *  Authors:
+ *    Lauris Kaplinski <lauris@ariman.ee>
+ *
+ *  Copyrithg (C) 1999-2000 authors
  *
  */
 
+#define GP_PATH_C
+
 #include <string.h>
 #include <libart_lgpl/art_misc.h>
 #include "gp-path.h"
@@ -29,9 +46,9 @@ struct _GPPath {
 	guint allopen : 1;	/* All subpaths are open */
 };
 
-static gboolean sp_bpath_good (ArtBpath * bpath);
-static ArtBpath * sp_bpath_check_subpath (ArtBpath * bpath);
-static gint sp_bpath_length (ArtBpath * bpath);
+static gboolean sp_bpath_good (const ArtBpath * bpath);
+static const ArtBpath * sp_bpath_check_subpath (const ArtBpath * bpath);
+static gint sp_bpath_length (const ArtBpath * bpath);
 static gboolean sp_bpath_all_closed (const ArtBpath * bpath);
 static gboolean sp_bpath_all_open (const ArtBpath * bpath);
 
@@ -460,7 +477,8 @@ gp_path_close_all (const GPPath * path)
 		}
 	}
 
-	if ((!closed) && ((start->x3 != p->x3) || (start->y3 != p->y3))) {
+	if (!closed && (p->code == ART_END ||
+			start->x3 != p->x3 || (start->y3 != p->y3))) {
 		d->code = ART_LINETO;
 		d->x3 = start->x3;
 		d->y3 = start->y3;
@@ -844,9 +862,9 @@ gp_path_all_closed (const GPPath * path)
 /* Private methods */
 
 static
-gboolean sp_bpath_good (ArtBpath * bpath)
+gboolean sp_bpath_good (const ArtBpath * bpath)
 {
-	ArtBpath * bp;
+	const ArtBpath * bp;
 
 	g_return_val_if_fail (bpath != NULL, FALSE);
 
@@ -862,8 +880,8 @@ gboolean sp_bpath_good (ArtBpath * bpath)
 	return TRUE;
 }
 
-static ArtBpath *
-sp_bpath_check_subpath (ArtBpath * bpath)
+static const ArtBpath *
+sp_bpath_check_subpath (const ArtBpath * bpath)
 {
 	gint i, len;
 	gboolean closed;
@@ -902,7 +920,7 @@ sp_bpath_check_subpath (ArtBpath * bpath)
 }
 
 static gint
-sp_bpath_length (ArtBpath * bpath)
+sp_bpath_length (const ArtBpath * bpath)
 {
 	gint l;
 
