@@ -274,42 +274,17 @@ gpa_list_add_child (GPAList *list, GPANode *child, GPANode *ref)
 	return TRUE;
 }
 
-#if 0
-gboolean
-gpa_list_test_item_by_id (GPAList *list, const gchar *id)
+gint
+gpa_list_get_length (GPAList *list)
 {
-	GPAItem *item;
+	GPANode *child;
+	gint num = 0;
 
-	g_return_val_if_fail (list != NULL, FALSE);
-	g_return_val_if_fail (GPA_IS_LIST (list), FALSE);
-	g_return_val_if_fail (id != NULL, FALSE);
-	g_return_val_if_fail (*id != '\0', FALSE);
-
-	for (item = list->items; item != NULL; item = item->next) {
-		if (item->id && !strcmp (item->id, id)) return TRUE;
+	child = list->children;
+	while (child) {
+		num ++;
+		child = child->next;
 	}
-
-	return FALSE;
+	
+	return num;
 }
-
-GPANode *
-gpa_list_get_item_by_id (GPAList *list, const gchar *id)
-{
-	GPAItem *item;
-
-	g_return_val_if_fail (list != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_LIST (list), NULL);
-	g_return_val_if_fail (id != NULL, NULL);
-	g_return_val_if_fail (*id != '\0', NULL);
-
-	for (item = list->items; item != NULL; item = item->next) {
-		if (item->id && !strcmp (item->id, id)) {
-			gpa_node_ref (GPA_NODE (item));
-			return GPA_NODE (item);
-		}
-	}
-
-	return NULL;
-}
-#endif
-
