@@ -33,6 +33,8 @@
 #include <libxml/parser.h>
 #include "gpa-node-private.h"
 #include "gpa-utils.h"
+#include "gpa-value.h"
+#include "gpa-reference.h"
 
 #define noGPA_CACHE_DEBUG
 #define noGPA_QUARK_DEBUG
@@ -350,4 +352,67 @@ gpa_quark_to_string (GPAQuark quark)
 	return NULL;
 }
 
+/**
+ * gpa_utils_dump_tree_with_level:
+ * @node: The node to dump its contents
+ * @level: How deep we are in the tree, used so that we know how many spaces to print
+ *         so that it really looks like a tree
+ * 
+ * Recursively prints a node and it childs.
+ **/
+static void
+gpa_utils_dump_tree_with_level (GPANode *node, gint level)
+{
+	GPANode *ref = NULL;
+	GPANode *child;
+	int i;
 
+	/* Print this leave indentation */
+	for (i = 0; i < level; i++)
+		g_print ("   ");
+
+	/* Print the object itself */
+	g_print ("%s [%s] (%d)", gpa_node_id (node),
+		 G_OBJECT_TYPE_NAME (node), GPOINTER_TO_INT (node));
+
+	if (strcmp (G_OBJECT_TYPE_NAME (node), "GPAReference") == 0) {
+		GPANode *tmp = GPA_REFERENCE (node)->ref;
+		g_print ("****");
+		if (tmp)
+			g_print ("     reference to a:%s\n", G_OBJECT_TYPE_NAME (tmp));
+
+		if (level > 1)   /* We follow references only on level 0 & 1 so that we */
+			return; /* see Global & Printer objects */
+	} else {
+		if (strcmp (G_OBJECT_TYPE_NAME (node), "GPAValue") == 0)
+			g_print ("{%s}", ((GPAValue *) node)->value);
+		g_print ("\n");
+	}
+
+	ref = NULL;
+	while (TRUE) {
+		child = gpa_node_get_child (node, ref);
+		if (!child)
+			break;
+		ref = child;
+		gpa_utils_dump_tree_with_level (child, level + 1);
+	}
+}
+
+/**
+ * gpa_utils_dump_tree:
+ * @node: 
+ * 
+ * Dump the tree pointed by @node to the console. Used for debuging purposeses
+ **/
+void
+gpa_utils_dump_tree (GPANode *node)
+{
+	g_print ("\n"
+		 "-------------\n"
+		 "Dumping a tree\n\n");
+
+	gpa_utils_dump_tree_with_level (node, 0);
+	
+	g_print ("-------------\n");
+}
