@@ -1,32 +1,34 @@
-#define __GPA_VENDOR_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * This file is part of libgnomeprint 2
+ *  gpa-vendor.c:
  *
- * Libgnomeprint is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
- * Libgnomeprint is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with the libgnomeprint; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Authors :
- *   Jose M. Celorio <chema@ximian.com>
- *   Lauris Kaplinski <lauris@ximian.com>
+ *  Authors :
+ *    Jose M. Celorio <chema@ximian.com>
+ *    Lauris Kaplinski <lauris@ximian.com>
  *
- * Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
+ *  Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
  *
  */
 
+#define __GPA_VENDOR_C__
+#define noGPA_VENDOR_DEBUG
+
 #include <string.h>
+#include <sys/types.h>
 #include <dirent.h> /* For the DIR structure stuff */
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
@@ -112,7 +114,8 @@ gpa_vendor_finalize (GObject *object)
 
 	vendor->name = gpa_node_detach_unref (GPA_NODE (vendor), vendor->name);
 	vendor->models = (GPAList *) gpa_node_detach_unref (GPA_NODE (vendor), GPA_NODE (vendor->models));
-	if (vendor->url) vendor->url = gpa_node_detach_unref (GPA_NODE (vendor), vendor->url);
+	if (vendor->url)
+		vendor->url = gpa_node_detach_unref (GPA_NODE (vendor), vendor->url);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -124,14 +127,21 @@ gpa_vendor_verify (GPANode *node)
 
 	vendor = GPA_VENDOR (node);
 
-	if (!GPA_NODE_ID_EXISTS (node)) return FALSE;
+	if (!GPA_NODE_ID_EXISTS (node))
+		return FALSE;
 
-	if (!vendor->name) return FALSE;
-	if (!gpa_node_verify (vendor->name)) return FALSE;
-	if (!vendor->url) return FALSE;
-	if (!gpa_node_verify (vendor->url)) return FALSE;
-	if (!vendor->models) return FALSE;
-	if (!gpa_node_verify (GPA_NODE (vendor->models))) return FALSE;
+	if (!vendor->name)
+		return FALSE;
+	if (!gpa_node_verify (vendor->name))
+		return FALSE;
+	if (!vendor->url)
+		return FALSE;
+	if (!gpa_node_verify (vendor->url))
+		return FALSE;
+	if (!vendor->models)
+		return FALSE;
+	if (!gpa_node_verify (GPA_NODE (vendor->models)))
+		return FALSE;
 
 	return TRUE;
 }
@@ -143,7 +153,8 @@ gpa_vendor_get_value (GPANode *node)
 
 	vendor = GPA_VENDOR (node);
 
-	if (GPA_NODE_ID_EXISTS (node)) return g_strdup (GPA_NODE_ID (node));
+	if (GPA_NODE_ID_EXISTS (node))
+		return g_strdup (GPA_NODE_ID (node));
 
 	return NULL;
 }
@@ -153,7 +164,7 @@ gpa_vendor_get_child (GPANode *node, GPANode *ref)
 {
 	GPAVendor *vendor;
 	GPANode *child;
-
+	
 	vendor = GPA_VENDOR (node);
 
 	child = NULL;
@@ -165,7 +176,8 @@ gpa_vendor_get_child (GPANode *node, GPANode *ref)
 		child = (GPANode *) vendor->models;
 	}
 
-	if (child) gpa_node_ref (child);
+	if (child)
+		gpa_node_ref (child);
 
 	return child;
 }
@@ -180,9 +192,12 @@ gpa_vendor_lookup (GPANode *node, const guchar *path)
 
 	child = NULL;
 
-	if (gpa_node_lookup_ref (&child, GPA_NODE (vendor->name), path, "Name")) return child;
-	if (gpa_node_lookup_ref (&child, GPA_NODE (vendor->models), path, "Models")) return child;
-	if (vendor->url && gpa_node_lookup_ref (&child, GPA_NODE (vendor->url), path, "URL")) return child;
+	if (gpa_node_lookup_ref (&child, GPA_NODE (vendor->name), path, "Name"))
+		return child;
+	if (gpa_node_lookup_ref (&child, GPA_NODE (vendor->models), path, "Models"))
+		return child;
+	if (vendor->url && gpa_node_lookup_ref (&child, GPA_NODE (vendor->url), path, "URL"))
+		return child;
 
 	return NULL;
 }
@@ -240,9 +255,12 @@ gpa_vendor_new_from_tree (xmlNodePtr tree)
 		vendor->url = gpa_node_attach (GPA_NODE (vendor), url);
 		vendor->models = (GPAList *) gpa_node_attach (GPA_NODE (vendor), GPA_NODE (models));
 	} else {
-		if (name) gpa_node_unref (name);
-		if (url) gpa_node_unref (url);
-		if (models) gpa_node_unref (GPA_NODE (models));
+		if (name)
+			gpa_node_unref (name);
+		if (url)
+			gpa_node_unref (url);
+		if (models)
+			gpa_node_unref (GPA_NODE (models));
 	}
 
 	xmlFree (xmlid);
@@ -258,7 +276,8 @@ gpa_vendor_new_from_file (const gchar *filename)
 	xmlNodePtr root;
 
 	doc = xmlParseFile (filename);
-	if (!doc) return NULL;
+	if (!doc)
+		return NULL;
 	root = doc->xmlRootNode;
 	vendor = NULL;
 	if (!strcmp (root->name, "Vendor")) {
@@ -272,25 +291,35 @@ gpa_vendor_new_from_file (const gchar *filename)
 
 static void gpa_vendor_list_load_from_dir (GPAList *vendors, const gchar *dirname);
 
+static GPAList *vendors = NULL;
+
+static void
+gpa_vendors_gone (gpointer data, GObject *gone)
+{
+#ifdef GPA_VENDOR_DEBUG
+	g_print ("GPAVendor: Vendor list %p has gone\n", gone);
+#endif
+
+	vendors = NULL;
+}
+
 GPAList *
 gpa_vendor_list_load (void)
 {
-	static GPAList *vendors = NULL;
+	gchar *dirname;
 
-	if (!vendors) {
-		gchar *dirname;
+	if (vendors)
+		return (GPAList *) gpa_node_ref (GPA_NODE (vendors));
 
-		vendors = GPA_LIST (gpa_list_new (GPA_TYPE_VENDOR, FALSE));
-		gpa_node_construct (GPA_NODE (vendors), "Vendors");
+	vendors = GPA_LIST (gpa_list_new (GPA_TYPE_VENDOR, FALSE));
+	gpa_node_construct (GPA_NODE (vendors), "Vendors");
+	g_object_weak_ref (G_OBJECT (vendors), gpa_vendors_gone, &vendors);
 
-		dirname = g_strdup_printf ("%s/%s", g_get_home_dir (), ".gnome/gnome-print-2.0/vendors");
-		gpa_vendor_list_load_from_dir (vendors, dirname);
-		g_free (dirname);
-		/* fixme: */
-		gpa_vendor_list_load_from_dir (vendors, DATADIR "/gnome-print-2.0/vendors");
-	}
-
-	gpa_node_ref (GPA_NODE (vendors));
+	dirname = g_strdup_printf ("%s/%s", g_get_home_dir (), ".gnome/gnome-print-2.0/vendors");
+	gpa_vendor_list_load_from_dir (vendors, dirname);
+	g_free (dirname);
+	/* fixme: */
+	gpa_vendor_list_load_from_dir (vendors, DATADIR "/gnome-print-2.0/vendors");
 
 	return vendors;
 }
@@ -315,9 +344,14 @@ gpa_vendor_list_load_from_dir (GPAList *vendors, const gchar *dirname)
 		if (strcmp (dent->d_name + len - 7, ".vendor")) continue;
 		filename = g_strdup_printf ("%s/%s", dirname, dent->d_name);
 		vendor = gpa_vendor_new_from_file (filename);
-		g_free (filename);
 		/* fixme: test name clashes */
-		if (vendor) l = g_slist_prepend (l, vendor);
+		if (vendor)
+			l = g_slist_prepend (l, vendor);
+		else
+			g_warning ("Could not create vendor from file:%s\n",
+				   filename);
+		g_free (filename);
+
 	}
 
 	closedir (dir);
@@ -348,12 +382,16 @@ gpa_vendor_get_by_id (const guchar *id)
 
 	for (child = vendors->children; child != NULL; child = child->next) {
 		g_assert (GPA_IS_VENDOR (child));
-		if (GPA_NODE_ID_COMPARE (child, id)) break;
+		if (GPA_NODE_ID_COMPARE (child, id))
+			break;
 	}
 
-	if (child) gpa_node_ref (child);
+	if (child)
+		gpa_node_ref (child);
+	else
+		g_print ("Could not get vendor by id: %s\n", id);
 
-	gpa_node_unref (GPA_NODE (vendors));
+	gpa_node_unref (gpa_node_cache (GPA_NODE (vendors)));
 
 	return child;
 }
